@@ -3,6 +3,7 @@ from .graph_style.NodeStyle import NodeStyle
 from .graph_style.EdgeStyle import EdgeStyle
 from .graph_style.RootAndBranchStylist import RootAndBranchStylist
 import graphviz
+from graphviz import Source, Graph as GraphvizGraph, Digraph
 import os
 from functools import wraps
 import random
@@ -268,7 +269,7 @@ class Graph(BasicGraph):
 	def get_graphviz_str(self, direction=None, dpi=300, height=None, width=None, pad=None):
 		"""
 		:type direction: str or NoneType
-		:type dpi: int
+		:type dpi: int or NoneType
 		:type height: float or int or NoneType
 		:type width: float or int or NoneType
 		:type pad: int or float or NoneType
@@ -307,6 +308,12 @@ class Graph(BasicGraph):
 			if 'ordering' in dictionary:
 				self._ordering = dictionary['ordering']
 
+			if 'label' in dictionary:
+				self._label = '\n' + dictionary['label']
+
+			if 'label_url' in dictionary:
+				self._label_url = dictionary['label_url']
+
 			if 'graph_node_style' in dictionary:
 				self._graph_node_style_overwrite = dictionary['graph_node_style']
 
@@ -534,18 +541,74 @@ class Graph(BasicGraph):
 		self.stylize()
 
 		if path is None:
-			return graphviz.Source(source=self.get_graphviz_str(direction=direction, pad=pad, dpi=None))
+			return self.get_graphviz_source(direction=direction, pad=pad, dpi=None)# Source(source=self.get_graphviz_str(direction=direction, pad=pad, dpi=None))
 		else:
 			filename, file_extension = os.path.splitext(path)
 			output_format = file_extension.lstrip('.')
-			graphviz_str_to_save = self.get_graphviz_str(direction=direction, height=height, width=width, pad=pad, dpi=dpi)
-			to_save = graphviz.Source(source=graphviz_str_to_save, format=output_format)
+			to_save = self.get_graphviz_source(format=output_format)
 			to_save.render(filename=filename, view=view)
-			graphviz_str_to_display = graphviz.Source(
-				source=self.get_graphviz_str(direction=direction, pad=pad, dpi=None)
-			)
+			graphviz_str_to_display = self.get_graphviz_source(direction=direction, pad=pad, dpi=None)
 			return graphviz_str_to_display
 
+	def get_graphviz_source(self, direction=None, dpi=300, height=None, width=None, pad=None, output_format=None):
+		"""
+		:type direction: NoneType or str
+		:type dpi: NoneType or int
+		:type height: NoneType of int or float
+		:type width: NoneType or int or float
+		:type pad: NoneType or int or float
+		:type output_format: str or NoneType
+		:rtype: Source
+		"""
+		if height is None and width is None:
+			return Source(source=self.get_graphviz_str(direction=direction, pad=pad, dpi=None))
+		else:
+			graphviz_str_to_save = self.get_graphviz_str(
+				direction=direction, height=height, width=width, pad=pad, dpi=dpi
+			)
+			return Source(source=graphviz_str_to_save, format=output_format)
+
+	def get_graphviz_graph(self, direction=None, dpi=300, height=None, width=None, pad=None, output_format=None):
+		"""
+		:type direction: NoneType or str
+		:type dpi: NoneType or int
+		:type height: NoneType of int or float
+		:type width: NoneType or int or float
+		:type pad: NoneType or int or float
+		:type output_format: str or NoneType
+		:rtype: GraphvizGraph or Digraph
+		"""
+		source = self.get_graphviz_source(
+			direction=direction, dpi=dpi, height=height, width=width, pad=pad, output_format=output_format
+		)
+		lst = str(source).splitlines()
+		has_comment = (lst[0].find('//') != -1)
+		is_direct_graph = False
+		skip_index = 0
+
+		if has_comment:
+			skip_index = 1
+			if lst[skip_index].find('graph {') != -1:
+				is_direct_graph = False
+		else:
+			if lst[0].find('graph {') != -1:
+				is_direct_graph = False
+
+		if is_direct_graph:
+			graph = Digraph()
+		else:
+			graph = GraphvizGraph()
+
+		graph.body.clear()
+
+		s = str()
+		for i in range(len(lst)):
+			if ((i > skip_index) and (i != len(lst) - 1)):
+				if has_comment:
+					graph.comment = lst[0].replace('//', '')
+				graph.body.append(lst[i])
+		return graph
+
 	def display(self, p=None, pad=0.2, direction=None, path=None, height=None, width=None, dpi=300):
 		try:
 			from IPython.core.display import display
